import docker
import time
from collections import Counter

client = docker.from_env()

esclaveId = 0

seuil_haut = 2
seuil_bas = 2
periode_de_verification = 10
compteur = Counter()
compteur['search'] = Counter()
compteur['found'] = Counter()


def getServiceBySubstring(service_partiel):
    services = client.services.list()
    for service in services:
        if service_partiel in service.name:
            return service

    print(f"Aucun conteneur trouvé contenant le nom '{service_partiel}'.")
    return None

def getConteneurBySubstring(conteneur_partiel):
    conteneurs = client.containers.list(all=True)
    for conteneur in conteneurs:
        if conteneur_partiel in conteneur.name:
            return conteneur

    print(f"Aucun conteneur trouvé contenant le nom '{conteneur_partiel}'.")
    return None


def obtenir_logs_conteneur(conteneur_partiel):
    application_conteneur = getConteneurBySubstring(conteneur_partiel)

    if application_conteneur:
        logs = application_conteneur.logs(since=(time.time() - periode_de_verification), until=time.time(), stdout=True, stderr=True)
        return logs.decode('utf-8')
    else:
        print(f"Aucun conteneur trouvé contenant le nom '{conteneur_partiel}'.")
        return ""


def compter_evenements(logs):
    for ligne in logs.split('\n'):
        if "search" in ligne:
            compteur['search'][esclaveId] += 1
        elif "found" in ligne:
            compteur['found'][esclaveId] += 1
            compteur['search'][esclaveId] -= 1
    return compteur


def ajuster_replicas(nom_du_service, delta):
    service = None
    try:
        service = client.services.get(nom_du_service)
    except docker.errors.NotFound:
        print(f"Le service {nom_du_service} n'existe pas.")
        return

    replicas_actuels = service.attrs['Spec']['Mode']['Replicated']['Replicas']
    nouveau_replicas = max(replicas_actuels + delta, 1)
    service.scale(nouveau_replicas)
    print(f"Nombre de réplicas ajusté à {nouveau_replicas} pour le service {nom_du_service}.")


def lister_services():
    services = client.services.list()
    for service in services:
        print(f"Nom du service : {service.name}")
        print(f"ID du service : {service.id}")
        print(f"Image utilisée : {service.attrs['Spec']['TaskTemplate']['ContainerSpec']['Image']}")
        print(f"Réplicas : {service.attrs['Spec']['Mode']['Replicated']['Replicas']}")


if __name__ == "__main__":
    nom_conteneur_partiel = "application"
    nom_du_service_a_replicater = getServiceBySubstring("hash_extractor")
    lister_services()
    while True:
        logs = obtenir_logs_conteneur(nom_conteneur_partiel)
        if logs:
            compteur = compter_evenements(logs)
            print(f"Occurrences (Search): {compteur['search']}, (Found): {compteur['found']}, (esclaveId): {esclaveId}, compteur: {compteur['search'][esclaveId]}")
            if compteur['search'][esclaveId] >= seuil_haut:
                ajuster_replicas(nom_du_service_a_replicater, 1)
                esclaveId += 1
            elif compteur['search'][esclaveId] == -1:
                if esclaveId > 0:
                    ajuster_replicas(nom_du_service_a_replicater, -1)
                    compteur['search'].pop(esclaveId, None)
                    esclaveId -= 1
            """elif compteur['found'][esclaveId] > seuil_bas:
                ajuster_replicas(nom_du_service_a_replicater, -1)
                esclaveId -= 1"""

        #lister_services()
        time.sleep(periode_de_verification)
